import threading


class LockedDict(dict):
    _lock: threading.Lock
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        
        self._lock = threading.Lock()
        
    def __enter__(self):
        self._lock.acquire()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._lock.release()
        
    
        
        
        
if __name__ == "__main__":
    cc = LockedDict()
    with cc:
        cc['test'] = "asdasdasd"
    
    print(cc)




import inspect
from hashlib import md5

__injection_data = {}


def get_hash(path: str):
    return md5(path.encode('utf-8')).hexdigest()

def mock_dependency(depend, newmock):
    path = inspect.getfile(depend)
    name = depend.__name__
    
    hash = f'{get_hash(path)}::{name}'
    hash = get_hash(hash)
    
    __injection_data[hash] = newmock


def get_dependency(depend):
    path = inspect.getfile(depend)
    name = depend.__name__
    
    hash = f'{get_hash(path)}::{name}'
    hash = get_hash(hash)
    
    hasil = __injection_data.get(hash)
    if hasil == None:
        hasil = depend()
        __injection_data[hash] = hasil
    
    return hasil

if __name__ == '__main__':
    
    class Repo:
        def __init__(self) -> None:
            print('init')
    
    def name():
        print('asdasd')
        return 'test'
    
    def get(close, name: str = get_dependency(Repo)):
        return name
    
    
    # get()
    # get()
    print('asdasd-----')
    hasil = inspect.signature(get).parameters['close']
    print(hasil)




import asyncio
import inspect
import functools
from contextvars import ContextVar, copy_context
from hashlib import md5
from collections import OrderedDict
import pickle

inject_context: ContextVar[dict] = ContextVar('Context injection nisa di')

def get_hash(path: str, *args, **kwargs):
    data = frozenset(OrderedDict(kwargs).items())
    return hash((path, *args, data))


def run_in_new_context(callable, *args, **kwarg):
    context = copy_context()
    
    if asyncio.iscoroutinefunction(callable):
        @functools.wraps(callable)
        async def wrapper(*warg, **wkwarg):
            
            inject_context.set({})
            return await callable(*warg, **wkwarg)
    else:
    
        @functools.wraps(callable)
        def wrapper(*warg, **wkwarg):
            
            inject_context.set({})
            return callable(*warg, **wkwarg)
    
    return context.run(wrapper, *args, **kwarg)


def get_context_dependency(depend, *args, **kwargs):
    injection_data = inject_context.get()
    
    
    path = inspect.getfile(depend)
    name = depend.__name__
    
    hash = f'{path}::{name}'
    hash = get_hash(hash, *args, **kwargs)
    
    hasil = injection_data.get(hash)
    if hasil == None:
        if asyncio.iscoroutinefunction(depend):
            hasil = asyncio.create_task(depend(*args, **kwargs))
        else:
            hasil = depend(*args, **kwargs)
        injection_data[hash] = hasil
    
    return hasil





class memorize(dict):
    def __init__(self, func):
        self.func = func

    def __call__(self, *args):
        
        return self[args]

    def __missing__(self, args):
        
        result = self[args] = self.func(*args)
        return result
    
    
    
if __name__ == '__main__':
    @memorize
    def test(c):
        print('running')
        return 'Blues'
    
    
    
    test(10)
    test(10)
    
    
    